#!/bin/bash

# initial setup of the neural network of pisces 1D. This is run once, at the start of the experiment.

# 1. Makes EXPID directory and copy all the content from EXP00 into a default folder (.xml, executable, namelists)
# 2. Reads the inputted latitude and longitude and makes 3*3 cropped netcdf files from the inputs, and places them in
#    a new directory
# 3. Run the simulation with the default parameters
# 4. Obtaining the bias between the model in default settings and the observed data, to be used as a reference for the
#    other simulations
# 5. Generates an array of randomly assigned parameters as an initial set and saves as a .csv file

mode=default  # observed or default
restart=F     # T or F. T = restart a stopped experiment; F = start a new one
crop=False

if [ $restart == 'F' ]
then

## EDIT HERE TO CHANGE THE LOCATION OF THE 1DPISCES
## DEFAULT: LAT=56, LON=-49 (LABRADOR SEA)
LAT=56
LON=-49

SCRHOME=/gpfs/scratch/bsc32/bsc32786/                                                        # base directory of the nemo model
MODEL=${SCRHOME}%EXPID%/EXP00/                                                               # path that will be created and the experiment stored
INPUTS=/gpfs/projects/bsc32/repository/pisces_off/ORCA1_OFF_PISCES_83138_2015/               # path to input (netCDF) files (PATH ONLY)
                # path to .xml, executable, and namelist files (PATH ONLY)
#NEMO=/gpfs/projects/bsc32/models/pisces_off/gapoc/cfgs/mP1D/EXP00/
NEMO=/gpfs/projects/bsc32/models/pisces_off/gapoc/cfgs/mP1D/EXP00/
VARIABLES=/gpfs/scratch/bsc32/bsc32786/pisces1d/parameters/variables9.csv  # path to the information about the variables (file must be called variables.csv) (PATH AND FILE)
#MODELFILE=PISCES_1m_19600101_19641231_ptrc_T.nc                                           # name of output file (FILE ONLY)
MODELFILE=PISCES_5d_19600101_19641231_ptrc_T.nc


if [ $mode = 'observed' ]
then
    # path to the observed data. (PATH AND FILE) MUST BE ON THE SAME GRID AS THE MODEL
    OBS=${SCRHOME}pisces1d/obs/Mprof_6901486_L75_5days_2015-01-01_2015-12-31reg.nc
elif [ $mode = 'default' ]
then
    OBS=NA
fi

### 1. MAKE EXPID DIRECTORY AND COPY ALL THE CONTENT FROM EXP00 INTO A DEFAULT FOLDER (.XML, EXECUTABLES, AND NAMELISTS)

mkdir -p ${MODEL}
cd ${MODEL}

mkdir -p bbp700
mkdir -p chla

rm -rf *
mkdir -p default
cd default
cp -R ${NEMO}nemo .
cp -R ${NEMO}namelist* .
cp -R ${NEMO}*.xml .
cd ..
cp ${VARIABLES} .

cd ${INPUTS}

# load appropriate modules
module load python
module load intel/2017.4
module load impi/2017.4
module load netcdf/4.4.1.1
module load hdf5/1.8.19

module load perl
module load udunits/2.2.25
module load gsl/2.4
module load nco

### 2. Reads the inputted latitude and longitude and makes 3*3 cropped netcdf files from the inputs, and places them in
### a new directory

if [ $crop = 'True' ]
then

function getindices {           # function that finds the nearest gridpoint to the entered coordinates, and returns
                                # their indices.
inds="$(python - $1 $2 << END

import numpy as np
from geopy import distance
import sys
from netCDF4 import Dataset
import os

def mindistcoords(lat, lon, latitudes, longitudes):
# finds the nearest index
    mindist = 20000
    minI = 0
    minJ = 0
    (x, y) = np.shape(latitudes)
    for I in range(len(latitudes[0])):
        for J in range(len(latitudes[:, 0])):
            if abs(lat - latitudes[J][I]) < 2 and abs(lon - longitudes[J][I]) < 2:
                dist = distance.distance((lat, lon), (latitudes[J][I], longitudes[J][I]))
                if dist < mindist:
                    mindist = dist
                    minI = I
                    minJ = J
    return minJ, minI

# variable grid can be any orca2 grid with T (centre of the cell)
grid = Dataset('dyna_grid_T.nc', 'r', format='NETCDF4_CLASSIC')
lons = np.array(grid.variables['nav_lon'])
lats = np.array(grid.variables['nav_lat'])

ilt,iln=mindistcoords(float(sys.argv[1]),float(sys.argv[2]),lats,lons)

print(ilt)
print(iln)

END
)"
}

getindices $LAT $LON

inds=(${inds})

# makes a new directory, crops the netcdf files and puts them there. If it already exists, it is replaced

cd ${INPUTS}
mkdir -p ./cropped

if find "./cropped" -mindepth 1 -print -quit 2>/dev/null | grep -q .; then
    rm ./cropped/*
fi

for file in *.nc
do
  echo cropping $file
  ncks -d x,$((inds[0]-1)),$((inds[0]+1)) -d y,$((inds[1]-1)),$((inds[1]+1)) ${file} cropped/${file}
done

cp -R ./cropped/* ${MODEL}default
mkdir ${MODEL}cropped
cp -R ./cropped/* ${MODEL}cropped

elif [ $crop = 'False' ]
then

cd ${MODEL}
mkdir -p ./cropped
cp -R ${INPUTS}* ${MODEL}default
cp -R ${INPUTS}* ${MODEL}cropped

fi
### 3.  run the simulation with the default parameters
module purge
module load intel/2017.4
module load impi/2018.4
module load netcdf/4.2
module load hdf5/1.8.19
module load perl

cd ${MODEL}default

mpirun -np 1 ./nemo

### 4. Obtaining the bias between the model in default settings and the observed data, to be used as a reference for the
###    other simulations

module load netcdf/4.4.1.1
module load mkl
module load python

function getstats {

stats="$(python - $1 $2 $3 << END
# $1 - path to the observed data, $2 - path to outputs

from netCDF4 import Dataset
import numpy as np
import os
import math

mode = '$3'

# extracting the observed data

def getobsdata(ds,varname):
    A = np.array(obs.variables[varname])
    A[A == 0.0] = np.nan
    A=np.log10(A)
    mn = np.nanmean(A)
    std = np.nanstd(A)
    A = (A - mn)/std
    return A, mn, std

def editmoddata(arr,d,s, mn=None, std=None):
    arr=arr[:,:np.shape(d)[0]]
    arr = arr[-s:]
    arr[arr == 200000039192547557376] = np.nan
    arr=np.log10(arr)
    if mn:
        arr = (arr - mn)/std
        return arr
    else:
        mn = np.nanmean(arr)
        std = np.nanmean(arr)
        arr = (arr - mn)/std
        return arr, mn, std

if mode == 'observed':
    obs=Dataset('$1')
    bbp700obs, bbp700obsmean, bbp700obsstd = getobsdata(obs,'bbp700_despiked')
    bPOCobs, bPOCobsmean, bPOCobsstd = getobsdata(obs,'bbp700_spike')
    chlaobs, chlaobsmean, chlaobsstd = getobsdata(obs,'chla_cal')
    depths=np.array(obs.variables['deptht'])
    S=np.shape(bbp700obs)[0]
    obs.close()

    # extracting the outputs of the default simulation
    mod = Dataset('$2')
    bbp700mod = (np.array(mod.variables['POC'])[-S*5:, :, 1, 1]+np.array(mod.variables['PHY'])[-S*5:, :, 1, 1])*0.001
    bPOCmod = (np.array(mod.variables['GOC'])[-S*5:, :, 1, 1]+np.array(mod.variables['PHY2'])[-S*5:, :, 1, 1])*0.001
    chlamod = (np.array(mod.variables['NCHL'])[-S*5:, :, 1, 1]+np.array(mod.variables['DCHL'])[-S*5:, :, 1, 1])
    mod.close()

    bbp700mod=editmoddata(bbp700mod,depths,S, bbp700obsmean, bbp700obsstd)
    bPOCmod=editmoddata(bPOCmod,depths,S, bPOCobsmean, bPOCobsstd)
    chlamod=editmoddata(chlamod,depths,S, chlaobsmean, chlaobsstd)

    bbp700obs=bbp700obs[:,30:]
    bPOCobs=bPOCobs[:,30:]
    chlaobs=chlaobs[:,:17]

    bbp700mod=bbp700mod[:,30:]
    bPOCmod=bPOCmod[:,30:]
    chlamod=chlamod[:,:17]

    # small poc stats

    mstd=np.nanstd(bbp700mod)#/np.nanstd(bbp700obs)
    R=float(np.ma.corrcoef(np.ma.masked_invalid(bbp700obs.flatten()), np.ma.masked_invalid(bbp700mod.flatten()))[0,1])
    RMSE=math.sqrt(np.nansum((bbp700obs-bbp700mod)**2)/bbp700obs.size)
    bbp700bias = np.nanmean(bbp700mod - bbp700obs)
    np.savetxt('bbp700defstats.csv',[bbp700bias,mstd,R,RMSE])

    mstd=np.nanstd(bPOCmod)#/np.nanstd(bPOCobs)
    R=float(np.ma.corrcoef(np.ma.masked_invalid(bPOCobs.flatten()), np.ma.masked_invalid(bPOCmod.flatten()))[0,1])
    bPOCbias = np.nanmean(bPOCmod - bPOCobs)
    RMSE=math.sqrt(np.nansum((bPOCobs-bPOCmod)**2)/bPOCobs.size)
    np.savetxt('bPOCdefstats.csv',[bPOCbias,mstd,R,RMSE])

elif mode == 'default':
    np.savetxt('bbp700defstats.csv',[0,1,1,0])
    np.savetxt('bPOCdefstats.csv',[0,1,1,0])

END
)"
}

getstats $OBS ${MODEL}default/$MODELFILE $mode

### 5. Generates an array of randomly assigned parameters as an initial set and saves as a .csv file

function makeparameters {

bias="$(python - $1 $2 $3 << END

import numpy as np
import random
import pandas

mode = '$3'
vars=pandas.read_csv('$2')

A=np.zeros((int($1),len(vars)))

for i in range(np.shape(A)[0]):
    for j in range(len(vars)):
        A[i,j]=random.random()*(vars['max'][j]-vars['min'][j])+vars['min'][j]
        #A[i,j]=vars['defaults'][j]
if mode == 'observed':
    A[0]=vars['defaults']
np.savetxt('0000.csv',A)

END
)"
}

cd $MODEL

mkdir -p bbp700
mkdir -p chla
mkdir -p parameters
cd parameters
if find . -mindepth 1 -print -quit 2>/dev/null | grep -q .; then
    rm *
fi

makeparameters %NUMCHUNKS% ${VARIABLES} $mode

fi
